<?php

namespace EventPortal\Http\Controllers\Admin\Event\Form;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Form\Actions\UpdateForm;
use EventPortal\Domain\Event\Form\Models\Form as FormModel;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Requests\UpdateFormRequest;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class FormController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:form.store')->only(['create', 'store']);
        $this->middleware('check-permission:form.update')->only(['edit', 'update']);
    }

    public function index(Event $event): Response
    {
        $table = Table::make('form.index')
            ->context($event)
            ->query(FormModel::forEvent($event))
            ->request(request())
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model(request()->event)
            ->render();

        $quickButton = Button::make('Create form')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.form.create', ['event' => $event->id]);

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Forms')
            ->get();

        return AdminRender::component('Admin/EventForm/Index')
            ->withTable($table)
            ->key('form.index')
            ->withSideBar($sidebar)
            ->withBreadcrumbs($breadcrumbs)
            ->withQuickButton($quickButton, has_permission('form.store'))
            ->get();
    }

    public function create(Event $event): Response
    {
        $form = Form::make('form.create')->model(new FormModel)->context($event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Create a form')
            ->get();

        return AdminRender::component('Admin/EventForm/Create')
            ->key('form.create')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle('Create a form')
            ->get();
    }

    public function edit(Event $event, FormModel $form): Response
    {
        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Forms', route('admin.form.index', [
                'event' => $event->id,
            ]))
            ->add('Edit a form')
            ->get();

        $data = $form->toArray();

        $data['type'] = Option::make($data['type'])->value($data['type'])->toArray();

        return AdminRender::component('Admin/EventForm/Edit')
            ->withForm($data)
            ->attributes([
                'routeData' => [
                    'route' => 'admin.form.update',
                    'routeParams' => [
                        'event' => $event->id,
                        'form' => $form->id,
                    ],
                    'method' => 'POST',
                ],
            ])
            ->withTitle('Edit a form')
            ->withBreadcrumbs($breadcrumbs)
            ->attributes([
                'formTypes' => FormModel::typesAsOptions(),
                'questionTypes' => Question::typesAsOptions(),
            ])
            ->get();
    }

    public function store(Request $request, Event $event): RedirectResponse
    {
        $payload = Form::make('form.create')
            ->model(new FormModel)
            ->context($event)
            ->validate($request->all())
            ->handle($request->all());

        toast()->success(trans('form.created'));

        return redirect()
            ->route('admin.form.edit', ['event' => $event, 'form' => $payload->getContext('form')]);
    }

    public function update(
        UpdateFormRequest $request,
        Event $event,
        FormModel $form,
        UpdateForm $updateForm
    ): RedirectResponse {
        $updateForm->execute($event, $form, $request->all());

        $form->touch();

        toast()->success(trans('form.updated'));

        return redirect()->route('admin.form.index', ['event' => $event]);
    }
}
