<?php

namespace EventPortal\Tests\Unit\Agenda;

use Carbon\Carbon;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Agenda\SessionInput;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class SessionInputTest extends TestCase
{
    use DatabaseMigrations;

    /** @var array */
    private $browserInput = [];

    /** @var array */
    private $databaseInput = [];

    public function setUp(): void
    {
        parent::setUp();

        $this->browserInput = [
            'session-name' => 'Breakfast!',
            'session-description' => 'Selection of breakfast foods.',
            'session-start-time' => [
                'startTime' => '08:00 AM',
            ],
            'session-end-time' => [
                'startTime' => '08:30 AM',
            ],
            'session-location' => 'Canteen',
        ];

        $this->databaseInput = [
            'name' => 'Breakfast!',
            'description' => 'Selection of breakfast foods.',
            'starting_at' => Carbon::parse('08:00'),
            'ending_at' => Carbon::parse('08:30'),
            'location' => 'Canteen',
        ];
    }

    /** @test **/
    public function can_create_from_browser_input()
    {
        $actual = SessionInput::from($this->browserInput);

        $this->assertInstanceOf(SessionInput::class, $actual);
        $this->assertEquals($this->browserInput, $actual->getInput());
    }

    /** @test **/
    public function can_create_from_database_input()
    {
        $actual = SessionInput::from($this->databaseInput);

        $this->assertInstanceOf(SessionInput::class, $actual);
        $this->assertEquals($this->databaseInput, $actual->getInput());
    }

    /** @test **/
    public function can_format_for_database()
    {
        Carbon::setTestNow('2020-07-01 15:00:00');

        $actual = SessionInput::from($this->browserInput)
            ->forceDate(Carbon::parse('2020-07-20'))
            ->formatForDatabase();

        $expected = [
            'name' => 'Breakfast!',
            'description' => 'Selection of breakfast foods.',
            'starting_at' => '2020-07-20 08:00:00',
            'ending_at' => '2020-07-20 08:30:00',
            'location' => 'Canteen',
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function can_format_for_database_with_unnested_start_and_end()
    {
        $input = [
            'session-name' => 'Breakfast!',
            'session-description' => 'Selection of breakfast foods.',
            'session-start-time' => '08:00 AM',
            'session-end-time' => '08:30 AM',
            'session-location' => 'Canteen',
        ];

        $expected = [
            'name' => 'Breakfast!',
            'description' => 'Selection of breakfast foods.',
            'starting_at' => '2020-07-20 08:00:00',
            'ending_at' => '2020-07-20 08:30:00',
            'location' => 'Canteen',
        ];

        $actual = SessionInput::from($input)->forceDate(Carbon::parse('2020-07-20'))->formatForDatabase();

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function can_format_for_browser()
    {
        $actual = SessionInput::from($this->databaseInput)->formatForBrowser();

        $expected = [
            'session-name' => 'Breakfast!',
            'session-description' => 'Selection of breakfast foods.',
            'session-start-time' => [
                'startTime' => '08:00 AM',
            ],
            'session-end-time' => [
                'startTime' => '08:30 AM',
            ],
            'session-location' => 'Canteen',
            'session-speakers' => [],
            'session-venue' => Option::make(''),
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function can_format_for_browser_with_session()
    {
        $session = factory(Session::class)->create();
        $data = array_merge($this->databaseInput, [
            'id' => $session->id,
        ]);
        $speaker = factory(Speaker::class)->create();
        $session->eventSpeakers()->create([
            'speaker_id' => $speaker->id,
            'event_id' => 1,
        ]);

        $actual = SessionInput::from($data)->formatForBrowser($session);

        $expected = [
            'session-name' => 'Breakfast!',
            'session-description' => 'Selection of breakfast foods.',
            'session-start-time' => [
                'startTime' => '08:00 AM',
            ],
            'session-end-time' => [
                'startTime' => '08:30 AM',
            ],
            'session-location' => 'Canteen',
            'session-speakers' => [
                Option::make($speaker->name)->value(1),
            ],
            'id' => 1,
            'session-venue' => Option::make(''),
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function can_format_for_browser_when_key_missing_from_map()
    {
        $expected = [
            'pathway_id' => 1,
            'session-speakers' => [],
            'session-venue' => Option::make(''),
        ];

        $this->assertEquals(
            $expected,
            SessionInput::from(['pathway_id' => 1])->formatForBrowser()
        );
    }
}
